#!/usr/bin/env groovy
package siena;
import siena.*;
import siena.dvdrp.*;

println("extractMatchingPercentages");

if (this.args.size() < 3){
  println("usage: extractMatchingPercentages mergedLogFileName resultFile aggregatedResultFile");
  return;
}
for (a in this.args) {
  println("Argument: " + a);
}

logFile = new File(args[0]);
filterFile = new File(args[1]);
filterFile.write('"MsgId"\t"Matching Filters %"\t"Matching BloomFilters %"\t"Matches"\t"BF Matches"\t"Filters"\n');
aggFile = new File(args[2]);
aggFile.write('"Percentage"\t"No. of Messages for Filters"\t"No. of Messages for Bloom Filters"\n');
List lines = logFile.readLines();
Map filters = new HashMap();
Map bFilters = new HashMap();
List percentages = [];
List bfPercentages = [];

//println(cmd);
pattern = /(\d+) (\w+) (\w+) .*\d: (\{.+\}){0,1}/;
for(i in 0..lines.size()-1){
    matcher = (lines[i] =~ pattern);
    //println(lines[i]);    
    if (matcher) {
        //println(matcher[0][0]);        
        int time = Integer.parseInt(matcher[0][1]);        
        switch (matcher[0][3]) {
            case "subscribing" :
                SENPBuffer pckt = new SENPBuffer(matcher[0][4].getBytes());
                Filter f = pckt.decodeFilter();
                List nodeFilters = filters.get(matcher[0][2]);
                if (nodeFilters == null){
                    nodeFilters = new ArrayList();
                    filters.put(matcher[0][2], nodeFilters);
                }
                //println("Filters for ${matcher[0][2]}: ${filters.get(matcher[0][2])}");
                // here we implement the same logic as in Siena
                // that is, if a subscription is covered by a more general one
                // only the most general is kept
                boolean toBeAdded = true;
                List toBeRemoved = [];
                for(filter in nodeFilters){
                    if (Covering.covers(f,filter)){
                        toBeRemoved.add(filter);                        
                    } else if (Covering.covers(filter,f)) {
                        toBeAdded = false;
                    }
                }
                // remove actual unneded filters
                for(filter in toBeRemoved){
                    nodeFilters.remove(filter);
                }
                if(toBeAdded){
                    nodeFilters.add(f);
                }
                //println("Filters for ${matcher[0][2]}: ${filters.get(matcher[0][2])}");
                // here we recompute BloomFilters for subscriber
                List nodeBFilters = bFilters.get(matcher[0][2]);
                if (nodeBFilters == null){
                    nodeBFilters = new ArrayList();
                    bFilters.put(matcher[0][2], nodeBFilters);
                }
                nodeBFilters.clear();
                for(filter in nodeFilters){
                    nodeBFilters.add(new BSet(filter));
                }
                break;   
            case "publishing" :
                int matches = 0;
                int bMatches = 0;
                int filtersCount = 0;
                //println("msg line: " + matcher[0][0]);                
                SENPBuffer pckt = new SENPBuffer(matcher[0][4].getBytes());
                Notification n = pckt.decodeNotification();
                String msgId = n.getAttribute("msgId").toString();
                //println("msgId: ${msgId}");
                // look for matching subscriptions                
                for(node in filters.keySet()){
                    //println(node);      
                    filtersCount += filters.get(node).size();
                    for(filter in filters.get(node)){                        
                        //println(filter);                        
                        if (Covering.apply(filter, n)){
                            matches++;
                        }
                    }
                }
                BSet msg = new BSet(n);
                for(node in bFilters.keySet()){
                    //println(node);                    
                    for(filter in bFilters.get(node)){                        
                        //println(filter);                        
                        if (msg.covers(filter)){
                            bMatches++;
                        }
                    }
                }
                filterFile.append(msgId + "\t" + matches / filtersCount + "\t" + bMatches / filtersCount  + "\t" + matches + "\t" + bMatches + "\t" + filtersCount + "\n");
                percentages.add(matches / filtersCount);
                bfPercentages.add(bMatches / filtersCount);
                break;
            case "unsubscribing" :                
                List nodeFilters = filters.get(matcher[0][2]);                
                SENPBuffer pckt = new SENPBuffer(matcher[0][4].getBytes());
                Filter f = pckt.decodeFilter();                
                nodeFilters.remove(f);                                
                // here we recompute BloomFilters for subscriber
                List nodeBFilters = bFilters.get(matcher[0][2]);
                if (nodeBFilters == null){
                    nodeBFilters = new ArrayList();
                    bFilters.put(matcher[0][2], nodeBFilters);
                }
                nodeBFilters.clear();
                for(filter in nodeFilters){
                    nodeBFilters.add(new BSet(filter));
                }
                break;           
        }
    }    
}
//aggregate data
println("percentages.size():" + percentages.size());
percentages = percentages.sort();
println(percentages);
Map aggPerc = new HashMap(); 
int i = 0;
int count = 0;
percentages.each(){
   if(it <= i){       
      count++; 
      //println(count);
   } else {
      aggPerc.put(i, count);
      while(i < it){
          i+= 0.1
      }
      count = 1;
   }
}
aggPerc.put(i, count);
println(aggPerc);
Map bfAggPerc = new HashMap();
bfPercentages = bfPercentages.sort();
println("bfPercentages.size():" + bfPercentages.size());
i = 0;
count = 0;
bfPercentages.each(){
   if(it <= i){       
      count++;            
   } else {
      bfAggPerc.put(i, count);
      while(i < it){
          i+= 0.1
      }
      count = 1;
   }
}
bfAggPerc.put(i, count);
println(bfAggPerc);
thresholds=[0,0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9,1];
for(k in thresholds){
    filter = (aggPerc.get(k) != null)? aggPerc.get(k) : 0;
    bFilter = (bfAggPerc.get(k) != null)? bfAggPerc.get(k) : 0;
    aggFile.append(k + "\t" + filter + "\t" + bFilter + "\n");
}

