// -*- C++ -*-
//
//  This file is part of WEEVIL.
//
//  Authors: See the file AUTHORS for full details.
//
//  Copyright (C) 2002-2006 University of Colorado
//
//  Weevil is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  Weevil is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
//  USA, or send email to serl@cs.colorado.edu.
//
// $Id: libweevil.cc,v 1.1.1.1 2004/11/02 23:36:42 ywang Exp $
//
#include <iostream>
#include <siena/ssim.h>
#include <cassert>
#include "weevil/weevil.h"

using namespace ssim;
using namespace std;
using namespace weevil;

map<string,ProcessId> WeevilGen::m_id2pid;
map<ProcessId,string> WeevilGen::m_pid2id;

void WeevilGen :: broadcast_event( const Event* event, Time lag )
{
  map<string,ProcessId>::const_iterator i;
  for( i = m_id2pid.begin(); i != m_id2pid.end(); ++i )
    {
      Sim::signal_event( (*i).second, event, lag );
    }
}

ProcessId WeevilGen :: id2pid( const string& id )
{
  map<string,ProcessId>::const_iterator itr = m_id2pid.find( id );
  assert( itr != m_id2pid.end() );
  return (*itr).second;
}

const string& WeevilGen :: pid2id( ProcessId pid )
{
  map<ProcessId,string>::const_iterator itr = m_pid2id.find( pid );
  assert( itr != m_pid2id.end() );
  return (*itr).second;
}

void WeevilGen :: self_signal_event( const Event* event, Time lag )
{
  Sim::self_signal_event( event, lag );
}

void WeevilGen :: signal_event( const string& id, const Event* event, Time lag )
{
  Sim::signal_event( id2pid( id ), event, lag );
}
